/// An immutable class to work as an option for methods in [PackagesInfoPlatform]
class Options {
  /// If this string is not empty, then only the package matching with this identifier will be returned.
  final String packageIdentifier;

  /// Will request the platform to include icons for package info.
  ///
  /// Defaults to true.
  final bool includeIcons;

  /// Will request the platform to include adaptable icons for package info.
  ///
  /// Defaults to false.
  final bool includeAdaptableIcons;

  const Options({
    this.includeIcons = true,
    this.includeAdaptableIcons = false,
    this.packageIdentifier = '',
  });

  Options copyWith({
    final String? packageIdentifier,
    final bool? includeIcons,
    final bool? includeAdaptableIcons,
  }) {
    return Options(
      includeIcons: includeIcons ?? this.includeIcons,
      includeAdaptableIcons:
          includeAdaptableIcons ?? this.includeAdaptableIcons,
      packageIdentifier: packageIdentifier ?? this.packageIdentifier,
    );
  }

  Map<String, dynamic> get toMap {
    return <String, dynamic>{
      'includeIcons': includeIcons,
      'includeAdaptableIcons': includeAdaptableIcons,
      'packageIdentifier': packageIdentifier,
    };
  }
}
