import 'package:flutter/foundation.dart';

enum PackageMapKey {
  displayName,
  packageIdentifier,
  version,
  buildNumber,
}

extension $PackageMapKey on PackageMapKey {
  String get name => describeEnum(this);
}

/// Package metadata. Represents an application bundle information on iOS and
/// application package information on Android.
class Package {
  Package({
    required this.displayName,
    required this.packageIdentifier,
    required this.version,
    required this.buildNumber,
  });

  /// The app name. `CFBundleDisplayName` on iOS, `application/label` on Android.
  final String displayName;

  /// The package name. `bundleIdentifier` on iOS, `getPackageName` on Android.
  final String packageIdentifier;

  /// The package version. `CFBundleShortVersionString` on iOS, `versionName` on Android.
  final String version;

  /// The build number. `CFBundleVersion` on iOS, `versionCode` on Android.
  final String buildNumber;

  factory Package.fromMap(Map packageMap) {
    return Package(
      displayName: packageMap[PackageMapKey.displayName.name] as String? ?? '',
      packageIdentifier:
          packageMap[PackageMapKey.packageIdentifier.name] as String? ?? '',
      version: packageMap[PackageMapKey.version.name] as String? ?? '',
      buildNumber: packageMap[PackageMapKey.buildNumber.name] as String? ?? '',
    );
  }

  @override
  bool operator ==(dynamic other) {
    if (other is! Package) return false;

    // Not testing Icon as it should only be different for different versions
    return displayName == other.displayName &&
        packageIdentifier == other.packageIdentifier &&
        version == other.version &&
        buildNumber == other.buildNumber;
  }

  @override
  int get hashCode =>
      displayName.hashCode +
      packageIdentifier.hashCode +
      version.hashCode +
      buildNumber.hashCode;
}
