import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart' show MethodChannel;

import 'method_channel_package_info.dart';
import 'option.dart';
import 'package.dart';

const String packagesInfoMethodChannelIdentifier =
    'plugins.purplegraphite.com/packages_info';

const MethodChannel packagesInfoMethodChannel =
    MethodChannel(packagesInfoMethodChannelIdentifier);

/// Application metadata provider. Provides application bundle information on iOS and
/// application package information on Android.
abstract class PackagesInfoPlatform {
  /// The default instance of [MethodChannelPackagesInfo] to use.
  static PackagesInfoPlatform _instance = MethodChannelPackagesInfo();

  /// The default instance of [PackagesInfoPlatform] to use.
  ///
  /// Defaults to [MethodChannelPackagesInfo]
  static PackagesInfoPlatform get instance => _instance;

  /// Platform-specific plugins should set this with their own platform-specific
  /// class that extends [PackagesInfoPlatform] when they register themselves.
  static set instance(PackagesInfoPlatform value) {
    if (!value.isMock) {
      try {
        value._verifyProvidesDefaultImplementations();
        // ignore: avoid_catching_errors
      } on NoSuchMethodError catch (_) {
        throw AssertionError(
            'Platform interfaces must not be implemented with `implements`');
      }
    }
    _instance = value;
  }

  /// Only mock implementations should set this to true.
  ///
  /// Mockito mocks are implementing this class with `implements` which is forbidden for anything
  /// other than mocks (see class docs). This property provides a backdoor for mockito mocks to
  /// skip the verification that the class isn't implemented with `implements`.
  @visibleForTesting
  bool get isMock => false;

  // This method makes sure that PackagesInfoPlatform isn't implemented with `implements`.
  //
  // See class doc for more details on why implementing this class is forbidden.
  //
  // This private method is called by the instance setter, which fails if the class is
  // implemented with `implements`.
  void _verifyProvidesDefaultImplementations() {}

  /// Return a List of all packages that are installed & available for the current user.
  Future<Iterable<Package>> getInstalledPackages([
    Options options = const Options(),
  ]);

  /// Retrieve overall information about a package that is installed on the system and is available to the current user.
  Future<Package> getPackageInfo(
    String packageIdentifier, [
    Options options = const Options(),
  ]);
}
